==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/binding/src/lib/actions/ServerAction.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from "@angular/core";
import type {
  RepetierState,
  ServerState, HardwareInfo, GPIO, WebCall, ExternalCommand, ExternalLink
} from "../state/RepetierState";
import {
  createAction, props, Store
} from "@ngrx/store";
import {take} from "rxjs/operators";
import {ConnectionHandler} from "../ConnectionHandler";
import {INITIAL_SERVER_FOLDER_STATE, Language, ServerFolderState} from "../state/RepetierState";
import {RSUtils, SerializationHelper} from "../RSUtils";

export const ServerSetLanguagesAction = createAction("SERVER_SET_LANGUAGES", props<{ languages: Language[] }>());
export const ServerSetExternalCommandsAction = createAction("SERVER_SET_EXTERNAL_COMMANDS", props<{ commands: ExternalCommand[] }>());
export const ServerSetWebCallsAction = createAction("SET_WEB_CALLS", props<{ webCalls: WebCall[] }>());
export const ServerSetGPIOAction = createAction("SET_GPIO", props<{ gpio: GPIO[] }>());
export const ServerSetAllExternalCommandsAction = createAction("SERVER_SET_ALL_EXTERNAL_COMMANDS", props<{ commands: ExternalCommand[] }>());
export const ServerSetGlobalAction = createAction("SERVER_SET_GLOBAL", props<{ key: string, data: any }>());
export const ServerInitSettingAction = createAction("SERVER_INIT_SETTING", props<{ key: string, data: any }>());
export const ServerUpdateSettingAction = createAction("SERVER_UPDATE_SETTING", props<{ key: string, data: any }>());
export const ServerSetServerFoldersAction = createAction("SERVER_SET_SERVER_FOLDERS", props<{ serverFolders: ServerFolderState[] }>());
export const ServerSetScreenLockAction = createAction("SERVER_SET_SCREEN_LOCK", props<{ lock: boolean }>());
export const ServerSetDiskUsageAction = createAction("SERVER_SET_DISK_USAGE", props<any>());
export const ServerSetHardwareInfoAction = createAction("SERVER_SET_HARDWARE_INFO", props<{ maxUrgency: number, list: HardwareInfo[] }>());
export const ServerSetExternalLinksAction = createAction("SERVER_SET_EXTERNAL_LINKS", props<{ links: ExternalLink[] }>());
export const ServerSetSettingsAction = createAction("SERVER_SET_SETTINGS", props<{ [key: string]: any }>());

@Injectable()
export class ServerAction {

  private openInits: any[] = [];

  constructor(private redux: Store<RepetierState>, private connectionHandler: ConnectionHandler) {
    this.openInits.push({
      key: "pricing", data: {
        currency: "EUR",
        perPrint: 5.0,
        perHour: 0.3,
        filaments: [{u: RSUtils.createAlphaID(10), n: "PLA", p: 25.0, w: 1.25}],
        map: {}
      }
    });
  }

  setScreenLock(lock: boolean): void {
    this.redux.dispatch(ServerSetScreenLockAction({lock}));
  }

  setLanguages(languages: Language[]): void {
    this.redux.dispatch(ServerSetLanguagesAction({languages}));
  }

  setExternalCommands(commands: ExternalCommand[]): void {
    this.redux.dispatch(ServerSetExternalCommandsAction({commands}));
  }

  setWebCalls(webCalls: WebCall[]): void {
    this.redux.dispatch(ServerSetWebCallsAction({webCalls}));
  }

  setGPIO(gpio: GPIO[]): void {
    this.redux.dispatch(ServerSetGPIOAction({gpio}));
  }

  setExternalLinks(links: ExternalLink[]): void {
    this.redux.dispatch(ServerSetExternalLinksAction({links}));
  }

  setAllExternalCommands(commands: ExternalCommand[]): void {
    this.redux.dispatch(ServerSetAllExternalCommandsAction({commands}));
  }

  setGlobalVar(key: string, data: any): void {
    this.redux.dispatch(ServerSetGlobalAction({key: key, data: data}));
  }

  get server(): ServerState {
    let pe: ServerState;
    this.redux.select("server").pipe(take(1)).subscribe(server => {
      pe = server;
    });
    return pe;
  }

  setSettings(data: { [key: string]: any }): void {
    if (RSUtils.newChanges(this.server.settings, data)) {
      this.redux.dispatch(ServerSetSettingsAction(data));
    }
  }

  initSetting(key: string, data: any): void {
    const server = this.server;
    if (server.settingsLoaded) {
      if (server.settings[key] === undefined) {
        const a: { [key: string]: any } = {};
        a[key] = data;
        this.setSettings(a); // immediate update for issuing device
        this.connectionHandler.send("updateSettings", a);
      } else {
        if (typeof data === "object") {
          let changed = false;
          const a: { [key: string]: any } = server.settings[key];
          Object.keys(data).forEach(dataKey => {
            if (a[dataKey] === undefined) {
              changed = true;
              a[dataKey] = data[dataKey];
            }
          });
          if (changed) {
            this.updateSetting(key, a);
          }
        }
      }
    } else if (!server.settingsLoaded) {
      this.openInits.push({key: key, data: data});
    }
  }

  updateSetting(key: string, data: any): Promise<any> {
    const server = this.server;
    if (server.settingsLoaded && !RSUtils.equals(server.settings[key], data)) {
      const a: { [key: string]: any } = {};
      a[key] = data;
      this.setSettings(a); // immediate update for issuing device
      return this.connectionHandler.send("updateSettings", a);
    }
    return Promise.resolve();
  }

  loadSettings() {
    this.connectionHandler.send("getSettings", {}).then((r) => {
      this.setSettings(r);
      for (let i = 0; i < this.openInits.length; i++) {
        this.initSetting(this.openInits[i].key, this.openInits[i].data);
      }
      this.openInits = [];
    });
  }

  updateDiskUsage() {
    this.connectionHandler.send("freeSpace", {}).then((r) => {
      this.redux.dispatch(ServerSetDiskUsageAction(r));
    });
  }

  runExternalCommand(cmd: ExternalCommand): Promise<any> {
    return this.connectionHandler.send("runExternalCommand", {id: cmd.id});
  }

  updateWebCalls(): Promise<any> {
    return this.connectionHandler.send("webCallsList", {}).then((r) => {
      this.setWebCalls(r.list);
    });
  }

  updateGPIO(): Promise<any> {
    return this.connectionHandler.send("GPIOGetList", {}).then((r) => {
      this.setGPIO(r.list);
    });
  }

  updateExternalLinks(): Promise<any> {
    return this.connectionHandler.send("getExternalLinks", {}).then((r) => {
      this.setExternalLinks(r.links);
    });
  }

  runWebCall(wc: WebCall): Promise<any> {
    return this.connectionHandler.send("webCallExecute", {name: wc.name});
  }

  updateHardwareInfo(data: { maxUrgency: number, list: HardwareInfo[] }): void {
    this.redux.dispatch(ServerSetHardwareInfoAction(data));
  }

  updateFolders() {
    this.connectionHandler.send("getFolders", {}).then((data) => {
      const folders: ServerFolderState[] = [];
      for (const f in data.folders) {
        if (data.folders.hasOwnProperty(f)) {
          folders.push(SerializationHelper.convert(Object.assign({}, INITIAL_SERVER_FOLDER_STATE), data.folders[f]));
        }
      }
      this.redux.dispatch(ServerSetServerFoldersAction({serverFolders: folders}));
    });
  }
}

==> RSTouchscreenNG-1.4.1.zip: RSTouchscreenNG/projects/binding/src/lib/actions/ServerAction.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from "@angular/core";
import type {
  RepetierState,
  ServerState, HardwareInfo, GPIO, WebCall, ExternalCommand, ExternalLink
} from "../state/RepetierState";
import {
  createAction, props, Store
} from "@ngrx/store";
import {take} from "rxjs/operators";
import {ConnectionHandler} from "../ConnectionHandler";
import {INITIAL_SERVER_FOLDER_STATE, Language, ServerFolderState} from "../state/RepetierState";
import {RSUtils, SerializationHelper} from "../RSUtils";

export const ServerSetLanguagesAction = createAction("SERVER_SET_LANGUAGES", props<{ languages: Language[] }>());
export const ServerSetExternalCommandsAction = createAction("SERVER_SET_EXTERNAL_COMMANDS", props<{ commands: ExternalCommand[] }>());
export const ServerSetWebCallsAction = createAction("SET_WEB_CALLS", props<{ webCalls: WebCall[] }>());
export const ServerSetGPIOAction = createAction("SET_GPIO", props<{ gpio: GPIO[] }>());
export const ServerSetAllExternalCommandsAction = createAction("SERVER_SET_ALL_EXTERNAL_COMMANDS", props<{ commands: ExternalCommand[] }>());
export const ServerSetGlobalAction = createAction("SERVER_SET_GLOBAL", props<{ key: string, data: any }>());
export const ServerInitSettingAction = createAction("SERVER_INIT_SETTING", props<{ key: string, data: any }>());
export const ServerUpdateSettingAction = createAction("SERVER_UPDATE_SETTING", props<{ key: string, data: any }>());
export const ServerSetServerFoldersAction = createAction("SERVER_SET_SERVER_FOLDERS", props<{ serverFolders: ServerFolderState[] }>());
export const ServerSetScreenLockAction = createAction("SERVER_SET_SCREEN_LOCK", props<{ lock: boolean }>());
export const ServerSetDiskUsageAction = createAction("SERVER_SET_DISK_USAGE", props<any>());
export const ServerSetHardwareInfoAction = createAction("SERVER_SET_HARDWARE_INFO", props<{ maxUrgency: number, list: HardwareInfo[] }>());
export const ServerSetExternalLinksAction = createAction("SERVER_SET_EXTERNAL_LINKS", props<{ links: ExternalLink[] }>());
export const ServerSetSettingsAction = createAction("SERVER_SET_SETTINGS", props<{ [key: string]: any }>());

@Injectable()
export class ServerAction {

  private openInits: any[] = [];

  constructor(private redux: Store<RepetierState>, private connectionHandler: ConnectionHandler) {
    this.openInits.push({
      key: "pricing", data: {
        currency: "EUR",
        perPrint: 5.0,
        perHour: 0.3,
        filaments: [{u: RSUtils.createAlphaID(10), n: "PLA", p: 25.0, w: 1.25}],
        map: {}
      }
    });
  }

  setScreenLock(lock: boolean): void {
    this.redux.dispatch(ServerSetScreenLockAction({lock}));
  }

  setLanguages(languages: Language[]): void {
    this.redux.dispatch(ServerSetLanguagesAction({languages}));
  }

  setExternalCommands(commands: ExternalCommand[]): void {
    this.redux.dispatch(ServerSetExternalCommandsAction({commands}));
  }

  setWebCalls(webCalls: WebCall[]): void {
    this.redux.dispatch(ServerSetWebCallsAction({webCalls}));
  }

  setGPIO(gpio: GPIO[]): void {
    this.redux.dispatch(ServerSetGPIOAction({gpio}));
  }

  setExternalLinks(links: ExternalLink[]): void {
    this.redux.dispatch(ServerSetExternalLinksAction({links}));
  }

  setAllExternalCommands(commands: ExternalCommand[]): void {
    this.redux.dispatch(ServerSetAllExternalCommandsAction({commands}));
  }

  setGlobalVar(key: string, data: any): void {
    this.redux.dispatch(ServerSetGlobalAction({key: key, data: data}));
  }

  get server(): ServerState {
    let pe: ServerState;
    this.redux.select("server").pipe(take(1)).subscribe(server => {
      pe = server;
    });
    return pe;
  }

  setSettings(data: { [key: string]: any }): void {
    if (RSUtils.newChanges(this.server.settings, data)) {
      this.redux.dispatch(ServerSetSettingsAction(data));
    }
  }

  initSetting(key: string, data: any): void {
    const server = this.server;
    if (server.settingsLoaded) {
      if (server.settings[key] === undefined) {
        const a: { [key: string]: any } = {};
        a[key] = data;
        this.setSettings(a); // immediate update for issuing device
        this.connectionHandler.send("updateSettings", a);
      } else {
        if (typeof data === "object") {
          let changed = false;
          const a: { [key: string]: any } = {...server.settings[key]};
          Object.keys(data).forEach(dataKey => {
            if (a[dataKey] === undefined) {
              changed = true;
              a[dataKey] = data[dataKey];
            }
          });
          if (changed) {
            this.updateSetting(key, a);
          }
        }
      }
    } else if (!server.settingsLoaded) {
      this.openInits.push({key: key, data: data});
    }
  }

  updateSetting(key: string, data: any): Promise<any> {
    const server = this.server;
    if (server.settingsLoaded && !RSUtils.equals(server.settings[key], data)) {
      const a: { [key: string]: any } = {};
      a[key] = data;
      this.setSettings(a); // immediate update for issuing device
      return this.connectionHandler.send("updateSettings", a);
    }
    return Promise.resolve();
  }

  loadSettings() {
    this.connectionHandler.send("getSettings", {}).then((r) => {
      this.setSettings(r);
      for (let i = 0; i < this.openInits.length; i++) {
        this.initSetting(this.openInits[i].key, this.openInits[i].data);
      }
      this.openInits = [];
    });
  }

  updateDiskUsage() {
    this.connectionHandler.send("freeSpace", {}).then((r) => {
      this.redux.dispatch(ServerSetDiskUsageAction(r));
    });
  }

  runExternalCommand(cmd: ExternalCommand): Promise<any> {
    return this.connectionHandler.send("runExternalCommand", {id: cmd.id});
  }

  updateWebCalls(): Promise<any> {
    return this.connectionHandler.send("webCallsList", {}).then((r) => {
      this.setWebCalls(r.list);
    });
  }

  updateGPIO(): Promise<any> {
    return this.connectionHandler.send("GPIOGetList", {}).then((r) => {
      this.setGPIO(r.list);
    });
  }

  updateExternalLinks(): Promise<any> {
    return this.connectionHandler.send("getExternalLinks", {}).then((r) => {
      this.setExternalLinks(r.links);
    });
  }

  runWebCall(wc: WebCall): Promise<any> {
    return this.connectionHandler.send("webCallExecute", {name: wc.name});
  }

  updateHardwareInfo(data: { maxUrgency: number, list: HardwareInfo[] }): void {
    this.redux.dispatch(ServerSetHardwareInfoAction(data));
  }

  updateFolders() {
    this.connectionHandler.send("getFolders", {}).then((data) => {
      const folders: ServerFolderState[] = [];
      for (const f in data.folders) {
        if (data.folders.hasOwnProperty(f)) {
          folders.push(SerializationHelper.convert(Object.assign({}, INITIAL_SERVER_FOLDER_STATE), data.folders[f]));
        }
      }
      this.redux.dispatch(ServerSetServerFoldersAction({serverFolders: folders}));
    });
  }
}
